//
//  AppDelegate.m
//  ADVNewsstandTemplate
//
//  Created by Tope on 07/03/2013.
//  Copyright (c) 2013 App Design Vault. All rights reserved.
//

#import "AppDelegate.h"
#import "BKRInterceptorWindow.h"
#import <Parse/Parse.h>
#import "Config.h"
#import <Fabric/Fabric.h>
#import <Crashlytics/Crashlytics.h>


@interface AppDelegate ()

@end


@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    [Fabric with:@[CrashlyticsKit]];
    
    // window to handle touches for html and hpub readers
    UIViewController *vc = self.window.rootViewController;
    self.window = [[BKRInterceptorWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.window.rootViewController = vc;
        
    [Parse setApplicationId:Config.parseAppId clientKey:Config.parseClientKey];
    
    
    // This tints the elements - settings, subscribe, close
    self.window.tintColor = [Config appNavigationBarTintColor];
    
    // Sets Navigation Bar Background Color
    [[UINavigationBar appearance] setBarTintColor:[Config appNavigationBarColor]];
    
    // Sets Navigation Bar title Color, Font and Size
    [[UINavigationBar appearance] setTitleTextAttributes: @{
                                                            NSForegroundColorAttributeName: [Config appNavigationBarTitleColor],
                                                            NSFontAttributeName: [Config appMainFont:21.0f]}
     ];

    
    UIProgressView.appearance.trackTintColor = [[Config appProgressViewColor] colorWithAlphaComponent:0.3];
    UIProgressView.appearance.progressTintColor = [Config appProgressViewColor];
    
    UIPageControl.appearance.tintColor = [[Config appPageControlColor] colorWithAlphaComponent:0.3];
    UIPageControl.appearance.currentPageIndicatorTintColor = [Config appPageControlColor];
    
    UIActivityIndicatorView.appearance.color = [Config appActivityIndicatorColor];
    
    
    if ([application respondsToSelector:@selector(registerUserNotificationSettings:)]) {
        UIUserNotificationType userNotificationTypes = (UIUserNotificationTypeAlert | UIUserNotificationTypeBadge | UIUserNotificationTypeSound);
        UIUserNotificationSettings *settings = [UIUserNotificationSettings settingsForTypes:userNotificationTypes categories:nil];
        [application registerUserNotificationSettings:settings];
        [application registerForRemoteNotifications];
        if (application.currentUserNotificationSettings.types & UIUserNotificationTypeBadge) {
            [[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];
        }
    }
    else {
        [application registerForRemoteNotificationTypes:(UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeAlert | UIRemoteNotificationTypeSound)];
        [[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];
    }
    
    if (application.applicationState != UIApplicationStateBackground) {
        // Track an app open here if we launch with a push, unless "content_available" was used to trigger a background push (introduced in iOS 7). In that case, we skip tracking here to avoid double counting the app-open.
        BOOL preBackgroundPush = ![application respondsToSelector:@selector(backgroundRefreshStatus)];
        BOOL oldPushHandlerOnly = ![self respondsToSelector:@selector(application:didReceiveRemoteNotification:fetchCompletionHandler:)];
        BOOL noPushPayload = ![launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
        if (preBackgroundPush || oldPushHandlerOnly || noPushPayload) {
            [PFAnalytics trackAppOpenedWithLaunchOptions:launchOptions];
        }
    }
    else {
        [PFAnalytics trackAppOpenedWithLaunchOptions:launchOptions];
    }
    
    sleep (3);
    return YES;
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    // Store the deviceToken in the current installation and save it to Parse.
    PFInstallation *currentInstallation = [PFInstallation currentInstallation];
    [currentInstallation setDeviceTokenFromData:deviceToken];
    [currentInstallation saveInBackground];
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo {
    [PFPush handlePush:userInfo];
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult result))handler {
    [[Publisher sharedInstance] getLatestInfo:^(NSError *error) {
        if (error == nil) {
            Issue *issue = nil;
            for (Issue *i in [Publisher sharedInstance].issues) {
                if ([i.uniqueId isEqualToString:userInfo[@"uniqueId"]]) {
                    issue = i;
                    break;
                }
            }
            [issue downloadIssue:^(NSError *error) {
                if (error == nil) {
                    handler(UIBackgroundFetchResultNewData);
                }
                else {
                    handler(UIBackgroundFetchResultFailed);
                }
            }];
        }
        else {
            handler(UIBackgroundFetchResultFailed);
        }
    }];
}

@end

