//
//  IssuePreviewController.m
//  Almond-AppVault
//
//  Created by Roman Stetsenko on 17/12/14.
//  Copyright (c) 2014 AkmStudio Ltd. All rights reserved.
//


#import "IssuePreviewController.h"
#import <Parse/Parse.h>
#import "Config.h"
#import "UIImageView+WebCache.h"


@interface IssuePreviewController () <UIScrollViewDelegate>

@property (nonatomic, weak) IBOutlet UILabel *titleLabel;
@property (nonatomic, weak) IBOutlet UIButton *actionButton;
@property (nonatomic, weak) IBOutlet UIButton *subscribeButton;
@property (nonatomic, weak) IBOutlet UITextView *descriptionTextView;

@property (nonatomic, weak) IBOutlet UIImageView *imgView1;
@property (nonatomic, weak) IBOutlet UIImageView *imgView2;
@property (nonatomic, weak) IBOutlet UIImageView *imgView3;
@property (nonatomic, weak) IBOutlet UIImageView *imgView4;

@property (nonatomic, weak) IBOutlet UIPageControl *pgControl;
@property (nonatomic, weak) IBOutlet UIProgressView *downloadProgressView;
@property (nonatomic, weak) IBOutlet UIActivityIndicatorView *downloadActivityView;

@property (weak, nonatomic) IBOutlet UINavigationBar *navBar;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *scrollAspectRatio;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *contentAspectRatio;

@end


@implementation IssuePreviewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [PFAnalytics trackEvent:([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad ? @"Preview_View_iPad" : @"Preview_View_iPhone")];
    
    [Config prepareButton:self.actionButton];
    [Config prepareButton:self.subscribeButton];
    
    self.titleLabel.textColor = [Config previewTitleColor];
    // AKM Added
    self.titleLabel.font = [Config appMainFont:16.0f];

    
    self.descriptionTextView.textColor = [Config previewDescriptionTextColor];
    self.descriptionTextView.selectable = NO;
    self.descriptionTextView.textContainerInset = UIEdgeInsetsMake(0, -5, 0, 0);
    // AKM Added
    self.descriptionTextView.font = [Config appMainFont:13.0f];
    
    [self updateView];
}

- (void)viewDidLayoutSubviews {
    [super viewDidLayoutSubviews];
    
    CGFloat scrollRatio = [Config issueImageAspectRatio];
    
    UIView *contentView = self.contentAspectRatio.firstItem;
    CGFloat contentRatio = (contentView.superview.bounds.size.width - 16.0) / contentView.superview.bounds.size.width;
    if ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad) {
        UIView *v = self.scrollAspectRatio.firstItem;
        contentRatio = v.frame.size.height*scrollRatio / contentView.superview.bounds.size.width;
    }
    
    NSLayoutConstraint *newConstraint1 = [NSLayoutConstraint constraintWithItem:self.contentAspectRatio.firstItem attribute:self.contentAspectRatio.firstAttribute relatedBy:self.contentAspectRatio.relation toItem:self.contentAspectRatio.secondItem attribute:self.contentAspectRatio.secondAttribute multiplier:contentRatio constant:self.contentAspectRatio.constant];
    UIView *view1 = self.contentAspectRatio.secondItem;
    [view1 removeConstraint:self.contentAspectRatio];
    [view1 addConstraint:newConstraint1];
    self.contentAspectRatio = newConstraint1;
    
    NSLayoutConstraint *newConstraint = [NSLayoutConstraint constraintWithItem:self.scrollAspectRatio.firstItem attribute:self.scrollAspectRatio.firstAttribute relatedBy:self.scrollAspectRatio.relation toItem:self.scrollAspectRatio.secondItem attribute:self.scrollAspectRatio.secondAttribute multiplier:scrollRatio constant:self.scrollAspectRatio.constant];
    UIView *view = self.scrollAspectRatio.firstItem;
    [view removeConstraint:self.scrollAspectRatio];
    [view addConstraint:newConstraint];
    self.scrollAspectRatio = newConstraint;
    
    self.descriptionTextView.contentOffset = CGPointZero;
    
    [self.view layoutSubviews];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    [self.descriptionTextView flashScrollIndicators];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)dealloc {
    [_issue removeObserver:self forKeyPath:@"status"];
    [_issue removeObserver:self forKeyPath:@"product.skProduct"];
    [_issue removeObserver:self forKeyPath:@"downloadProgress"];
}

#pragma mark -
- (void)setIssue:(Issue *)issue {
    [_issue removeObserver:self forKeyPath:@"status"];
    [_issue removeObserver:self forKeyPath:@"product.skProduct"];
    [_issue removeObserver:self forKeyPath:@"downloadProgress"];
    
    _issue = issue;
    if (self.isViewLoaded) {
        [self updateView];
    }
    
    [_issue addObserver:self forKeyPath:@"status" options:NSKeyValueObservingOptionNew context:nil];
    [_issue addObserver:self forKeyPath:@"product.skProduct" options:NSKeyValueObservingOptionNew context:nil];
    [_issue addObserver:self forKeyPath:@"downloadProgress" options:NSKeyValueObservingOptionNew context:nil];
}

- (void)updateView {
    self.navBar.topItem.title = [Config magazineName];
    self.navBar.backgroundColor = [UIColor blackColor];
    
    self.titleLabel.text = self.issue.title;
    self.descriptionTextView.text = self.issue.descriptionText;
    
    if (self.issue.previewImageURLs.count >= 4) {
        [self.imgView1 sd_setImageWithURL:self.issue.previewImageURLs[0] placeholderImage:nil];
        [self.imgView2 sd_setImageWithURL:self.issue.previewImageURLs[1] placeholderImage:nil];
        [self.imgView3 sd_setImageWithURL:self.issue.previewImageURLs[2] placeholderImage:nil];
        [self.imgView4 sd_setImageWithURL:self.issue.previewImageURLs[3] placeholderImage:nil];
    }
    
    if (self.issue.status == IssueStatusReady) {
        [self.actionButton setTitle:@"Read" forState:UIControlStateNormal];
        self.actionButton.hidden = NO;
        self.downloadProgressView.hidden = YES;
        self.downloadActivityView.hidden = YES;
    }
    else if (self.issue.status == IssueStatusDownloading) {
        [self.actionButton setTitle:@"" forState:UIControlStateNormal];
        self.actionButton.hidden = YES;
        self.downloadProgressView.hidden = NO;
        self.downloadActivityView.hidden = NO;
        self.downloadProgressView.progress = self.issue.downloadProgress;
    }
    else if (self.issue.status == IssueStatusNotPurchased) {
        self.actionButton.hidden = NO;
        self.downloadProgressView.hidden = YES;
        self.downloadActivityView.hidden = YES;
        if (self.issue.product.skProduct != nil) {
            NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
            [numberFormatter setFormatterBehavior:NSNumberFormatterBehaviorDefault];
            [numberFormatter setNumberStyle:NSNumberFormatterCurrencyStyle];
            [numberFormatter setLocale:self.issue.product.skProduct.priceLocale];
            
            NSString *price = [numberFormatter stringFromNumber:self.issue.product.skProduct.price];
            [self.actionButton setTitle:[NSString stringWithFormat:@"%@", price] forState:UIControlStateNormal];
            
        }
        else {
            [self.actionButton setTitle:@"Buy" forState:UIControlStateNormal];
        }
    }
    else if (self.issue.status == IssueStatusNotDownloaded) {
        [self.actionButton setTitle:@"Download" forState:UIControlStateNormal];
        self.actionButton.hidden = NO;
        self.downloadProgressView.hidden = YES;
        self.downloadActivityView.hidden = YES;
    }
}

- (IBAction)subscribeButtonTapped {
    RUN_BLOCK_SAFE(self.subscribeBlock);
}

- (IBAction)actionButtonTapped {
    RUN_BLOCK_SAFE(self.actionBlock, self.issue);
}

- (IBAction)closeButtonTapped {
    [self.parentViewController performSelector:@selector(dismissPreview) withObject:nil];
}

#pragma mark - UIScrollViewDelegate
- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView {
    self.pgControl.currentPage = round(scrollView.contentOffset.x / scrollView.bounds.size.width);
}

#pragma mark - KVO
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
    dispatch_async(dispatch_get_main_queue(), ^{
        if ([keyPath isEqualToString:@"status"] || [keyPath isEqualToString:@"product.skProduct"]) {
            [self updateView];
        }
        else if ([keyPath isEqualToString:@"downloadProgress"]) {
            self.downloadProgressView.progress = self.issue.downloadProgress;
        }
    });
}

@end

