//
//  IssueCellBase.m
//  ADVNewsstand
//
//  Created by Tope Abayomi on 24/09/2013.
//  Copyright (c) 2013 App Design Vault. All rights reserved.
//


#import "IssueCell.h"
#import "UIImageView+WebCache.h"
#import "Config.h"


@interface IssueCell ()

@property (nonatomic, weak) IBOutlet UIImageView *coverImageView;
@property (nonatomic, weak) IBOutlet UILabel *issueTitleLabel;
@property (nonatomic, weak) IBOutlet UIButton *actionButton;
@property (nonatomic, weak) IBOutlet UIProgressView *downloadProgressView;
@property (nonatomic, weak) IBOutlet UIActivityIndicatorView *activityView;

@end


@implementation IssueCell

- (void)awakeFromNib {
    [Config prepareButton:self.actionButton];
    [self.actionButton addTarget:self action:@selector(actionTapped) forControlEvents:UIControlEventTouchUpInside];
    
    [self.issueTitleLabel setFont:[Config appMainFont:15.0]];
    self.issueTitleLabel.textColor = [Config issueTitleLabelColor];
    
    self.layer.borderWidth = 0.0;
    self.layer.borderColor = [UIColor lightGrayColor].CGColor;
    self.layer.backgroundColor = [Config issueCellBackgroundColor].CGColor;
}

- (void)dealloc {
    [_issue removeObserver:self forKeyPath:@"status"];
    [_issue removeObserver:self forKeyPath:@"product.skProduct"];
    [_issue removeObserver:self forKeyPath:@"downloadProgress"];
}

#pragma mark -
- (void)setIssue:(Issue *)issue {
    if ([_issue isEqual:issue]) {
        // Nothing to do
        return;
    }
    
    [_issue removeObserver:self forKeyPath:@"status"];
    [_issue removeObserver:self forKeyPath:@"product.skProduct"];
    [_issue removeObserver:self forKeyPath:@"downloadProgress"];
    
    _issue = issue;
    [self updateView];
    
    [_issue addObserver:self forKeyPath:@"status" options:NSKeyValueObservingOptionNew context:nil];
    [_issue addObserver:self forKeyPath:@"product.skProduct" options:NSKeyValueObservingOptionNew context:nil];
    [_issue addObserver:self forKeyPath:@"downloadProgress" options:NSKeyValueObservingOptionNew context:nil];
}

- (void)updateView {
    if (self.issue == nil) {
        self.coverImageView.image = nil;
        self.issueTitleLabel.text = @"";
        self.actionButton.hidden = YES;
        self.downloadProgressView.hidden = YES;
        self.activityView.hidden = YES;
        return;
    }
    
    [self.coverImageView sd_setImageWithURL:self.issue.coverImageUrl];
    self.issueTitleLabel.text = self.issue.title;
    self.downloadProgressView.progress = self.issue.downloadProgress;
    [self updateIssueStatus];
}

- (void)updateIssueStatus {
    IssueStatus status = self.issue.status;
    
    if (status == IssueStatusReady) {
        [self.actionButton setTitle:@"Read" forState:UIControlStateNormal];
        self.actionButton.hidden = NO;
        self.downloadProgressView.hidden = YES;
        self.activityView.hidden = YES;
    }
    else if (status == IssueStatusDownloading) {
        [self.actionButton setTitle:@"" forState:UIControlStateNormal];
        self.actionButton.hidden = YES;
        self.downloadProgressView.hidden = NO;
        self.activityView.hidden = NO;
    }
    else if (status == IssueStatusNotPurchased) {
        self.actionButton.hidden = NO;
        self.downloadProgressView.hidden = YES;
        self.activityView.hidden = YES;
        
        if (self.issue.product.skProduct != nil) {
            NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
            [numberFormatter setFormatterBehavior:NSNumberFormatterBehaviorDefault];
            [numberFormatter setNumberStyle:NSNumberFormatterCurrencyStyle];
            [numberFormatter setLocale:self.issue.product.skProduct.priceLocale];
            
            
            NSString *price = [numberFormatter stringFromNumber:self.issue.product.skProduct.price];
            [self.actionButton setTitle:[NSString stringWithFormat:@"%@", price] forState:UIControlStateNormal];
            
        }        
        else {
            [self.actionButton setTitle:@"Buy" forState:UIControlStateNormal];
        }
    }
    else if (status == IssueStatusNotDownloaded) {
        [self.actionButton setTitle:@"Download" forState:UIControlStateNormal];
        self.actionButton.hidden = NO;
        self.downloadProgressView.hidden = YES;
        self.activityView.hidden = YES;
    }
    else {
        NSLog(@"WARNING: Unsupported issue status: %lu", (long)status);
    }
}

#pragma mark - Actions
- (void)actionTapped {
    RUN_BLOCK_SAFE(self.actionBlock, self.issue);
}

#pragma mark - KVO
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
    dispatch_async(dispatch_get_main_queue(), ^{
        if ([keyPath isEqualToString:@"status"] || [keyPath isEqualToString:@"product.skProduct"]) {
            [self updateIssueStatus];
        }
        else if ([keyPath isEqualToString:@"downloadProgress"]) {
            self.downloadProgressView.progress = self.issue.downloadProgress;
        }
    });
}

@end

