//
//  ViewController.m
//  ADVNewsstandTemplate
//
//  Created by Tope on 07/03/2013.
//  Copyright (c) 2013 App Design Vault. All rights reserved.
//


#import "IssuesGridViewController.h"
#import <Parse/Parse.h>
#import "Config.h"

#import "IssuePreviewController.h"
#import "IssueCell.h"
#import "IssueCellHeader.h"
#import "MBProgressHUD.h"

#import "MagazinePageViewController.h"
#import "BKRBookViewController.h"
#import "PDFReaderViewController.h"


static const NSInteger tAlertDeleteIssues = 243;


@interface IssuesGridViewController () <UICollectionViewDataSource, UICollectionViewDelegateFlowLayout, UIActionSheetDelegate, UIAlertViewDelegate>

@property (nonatomic, weak) IBOutlet UICollectionView *collectionView;
@property (nonatomic, weak) IBOutlet UICollectionViewFlowLayout *layout;

@property (nonatomic, strong) NSArray *menuOptions;
@property (nonatomic, strong) NSDate *lastUpdateDate;
@property (nonatomic, assign) BOOL updateInProgress;

@end


@implementation IssuesGridViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // This tracks how many times the issue index page is viewed on iPhone & iPad
    [PFAnalytics trackEvent:(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? @"Issues_View_iPad" : @"Issues_View_iPhone")];

    
    self.title = [Publisher sharedInstance].magazineName;
    
    self.menuOptions = @[//@{@"title":@"Subscribe to Magazine", @"action":NSStringFromSelector(@selector(subscribeToMagazine))},
                         @{@"title":@"Delete All Issues", @"action":NSStringFromSelector(@selector(deleteAllIssues))},
                         @{@"title":@"Restore All Purchases", @"action":NSStringFromSelector(@selector(restorePurchases))}];
    [self updateData];
        
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appBecomeActive:) name:UIApplicationDidBecomeActiveNotification object:nil];
    
    [self.navigationController.navigationBar setBackgroundImage:[[UIImage alloc] init] forBarMetrics:UIBarMetricsDefault];
    self.navigationController.navigationBar.shadowImage = [[UIImage alloc] init];
    
    self.collectionView.bounces = NO;

}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark -
- (void)updateView {
    self.title = [Publisher sharedInstance].magazineName;
    [self.collectionView reloadData];
}

- (void)updateData {
    if (self.updateInProgress) {
        return;
    }
    
    self.updateInProgress = YES;
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[UIApplication sharedApplication].delegate.window animated:NO];
    hud.labelText = @"Fetching new data...";
    [[Publisher sharedInstance] getLatestInfo:^(NSError *error) {
        if (error != nil) {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:error.localizedDescription delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
            [self updateView];
        }
        else {
            self.lastUpdateDate = [NSDate date];
            [self updateView];
        }
        [MBProgressHUD hideHUDForView:[UIApplication sharedApplication].delegate.window animated:YES];
        self.updateInProgress = NO;
    }];
}

- (IBAction)showOptionsMenu {
    UIActionSheet *sheet = [[UIActionSheet alloc] initWithTitle:nil delegate:self cancelButtonTitle:@"Close" destructiveButtonTitle:nil otherButtonTitles:nil];
    for (NSDictionary *dict in self.menuOptions) {
        [sheet addButtonWithTitle:dict[@"title"]];
        
    }
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
        [sheet showFromBarButtonItem:self.navigationItem.rightBarButtonItem animated:YES];
    }
    else {
        [sheet showInView:self.view];
    }
    
    // UIActionSheet Customisation
    SEL selector = NSSelectorFromString(@"_alertController");
    if ([sheet respondsToSelector:selector])
    {
        UIAlertController *alertController = [sheet valueForKey:@"_alertController"];
        if ([alertController isKindOfClass:[UIAlertController class]])
        {
            alertController.view.tintColor = [Config appMainColor];
//            [alertController.view setTintColor:[UIColor redColor]];
            // alertController.view.backgroundColor = [Config appNavigationBarColor]; // Changes color behind the cells
        }
    }
    else
    {
        // use other methods for iOS 7 or older.
    }
}

- (IBAction)subscribeToMagazine {
    [PFAnalytics trackEvent:([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad ? @"SubscribeBtn_iPad" : @"SubscribeBtn_iPhone")];
    
    if ([[Publisher sharedInstance] isSubscribedToMagazine:nil]) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Whoops" message:@"You are already subscribed" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        return;
    }
    
    if ([Publisher sharedInstance].subscriptions.count == 0) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Whoops" message:@"Subscription is not available for this magazine" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        return;
    }
    
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.navigationController.view.window animated:YES];
    hud.labelText = @"Subscribing...";
    
//    if ([Publisher sharedInstance].subscriptions.count > 1) {
#warning RS: add support for multiple subscribe options
//        [MBProgressHUD hideHUDForView:self.view.window animated:YES];
//    }
//    else
    {
        IAPProduct *product = [Publisher sharedInstance].subscriptions[0];
        [[Publisher sharedInstance] subscribe:product completion:^(NSError *error) {
            if (error != nil) {
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:error.localizedDescription delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
                [alert show];
            }
            else {
                [self updateView];
            }
            [MBProgressHUD hideHUDForView:self.navigationController.view.window animated:YES];
        }];
    }
}

- (void)deleteAllIssues {
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Are you sure?" message:@"Do you really want to remove all downloaded Issues?" delegate:self cancelButtonTitle:@"NO" otherButtonTitles:@"YES", nil];
    alertView.tag = tAlertDeleteIssues;
    [alertView show];
}

- (void)restorePurchases {
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view.window animated:YES];
    hud.labelText = @"Restoring...";
    [[Publisher sharedInstance] restorePurchases:^(NSError *error) {
        if (error != nil) {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:error.localizedDescription delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
        }
        else {
            [self updateView];
        }
        [MBProgressHUD hideHUDForView:self.view.window animated:YES];
    }];
}

#pragma mark -
- (void)handleIssueAction:(Issue *)issue {
    switch (issue.status) {
        case IssueStatusNotPurchased:
            [self purchaseIssue:issue];
            break;
        case IssueStatusNotDownloaded:
            [self downloadIssue:issue];
            break;
        case IssueStatusReady:
            [self readIssue:issue];
            break;
        default:
            [self previewIssue:issue];
            break;
    }
}

- (void)previewIssue:(Issue *)issue {
    self.view.window.userInteractionEnabled = NO;
    
    IssuePreviewController *previewVC = [self.storyboard instantiateViewControllerWithIdentifier:@"IssuePreviewController"];
    previewVC.issue = issue;
    previewVC.actionBlock = ^ (Issue *issue) {
        [self dismissPreview];
        [self handleIssueAction:issue];
    };
    previewVC.subscribeBlock = ^{
        [self dismissPreview];
        [self subscribeToMagazine];
    };
    
    UIView *v = [[UIView alloc] initWithFrame:self.view.window.frame];
    v.backgroundColor = [Config underiOS8BackgroundColor];
    v.tag = 263;
    
    if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"8.0")) {
        v.backgroundColor = [UIColor clearColor];
        UIBlurEffect *blurEffect = [Config appPreviewBlurEffect];
        UIVisualEffectView *blurEffectView = [[UIVisualEffectView alloc] initWithEffect:blurEffect];
        [blurEffectView setFrame:v.bounds];
        [v addSubview:blurEffectView];
    }
    
    UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(dismissPreview)];
    [v addGestureRecognizer:tapGR];
    [self.view.window addSubview:v];
    
    float propX = 1.0;
    float propY = 1.0;
    
    previewVC.view.frame = CGRectMake(v.bounds.size.width*(1.0-propX)/2.0, v.bounds.size.height*(1.0-propY)/2.0, v.bounds.size.width*propX, v.bounds.size.height*propY);
    [v addSubview:previewVC.view];
    [self addChildViewController:previewVC];
    
    NSUInteger index = [[Publisher sharedInstance].issues indexOfObjectIdenticalTo:issue];
    
    UICollectionViewCell *cell = [self.collectionView cellForItemAtIndexPath:[NSIndexPath indexPathForRow:index inSection:0]];
    CGRect r = [self.view.window convertRect:cell.bounds fromView:cell];
    v.transform = CGAffineTransformConcat(CGAffineTransformMakeScale(0.01, 0.01), CGAffineTransformMakeTranslation(r.origin.x + r.size.width/2.0 - self.view.window.bounds.size.width/2.0, r.origin.y + r.size.height/2.0 - self.view.window.bounds.size.height/2.0));
    
    [UIView animateWithDuration:0.3 animations:^{
        v.transform = CGAffineTransformIdentity;
    } completion:^(BOOL finished) {
        [previewVC didMoveToParentViewController:self];
        self.view.window.userInteractionEnabled = YES;
    }];
}

- (void)dismissPreview {
    UIView *v = [self.view.window viewWithTag:263];
    [self.childViewControllers[0] willMoveToParentViewController:nil];
    [UIView animateWithDuration:0.3 animations:^{
        v.transform = CGAffineTransformMakeScale(0.01, 0.01);
    } completion:^(BOOL finished) {
        [v removeFromSuperview];
        [self.childViewControllers[0] removeFromParentViewController];
    }];
}

- (void)purchaseIssue:(Issue *)issue {
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view.window animated:YES];
    hud.labelText = @"Purchasing...";
    [[Publisher sharedInstance] purchaseIssue:issue completion:^(NSError *error) {
        if (error != nil) {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:error.localizedDescription delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
        }
        else {
            [self updateView];
        }
        [MBProgressHUD hideHUDForView:self.view.window animated:YES];
    }];
}

- (void)downloadIssue:(Issue *)issue {
    [issue downloadIssue:^(NSError *error) {
        if (error != nil) {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:error.localizedDescription delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
        }
    }];
}

- (void)readIssue:(Issue *)issue {
    if ([Config isNewsstandApp]) {
        [[NKLibrary sharedLibrary] setCurrentlyReadingIssue:issue.nkIssue];
    }
    
    if (issue.type == IssueTypePDF) {
        PDFReaderViewController *readerViewController = [[PDFReaderViewController alloc] initWithIssue:issue];
        [self.navigationController presentViewController:readerViewController animated:YES completion:nil];
    }
    else if (issue.type == IssueTypeHPUB) {
        NSString *manifestFile = [issue.downloadedPath stringByAppendingPathComponent:@"book.json"];
        if ([[NSFileManager defaultManager] fileExistsAtPath:manifestFile]) {
            BKRBook *book = [[BKRBook alloc] initWithBookPath:issue.downloadedPath bundled:YES];
            if (book) {
                BKRBookViewController *bakerViewController = [[BKRBookViewController alloc] initWithBook:book];
                [self.navigationController pushViewController:bakerViewController animated:YES];
//                [self.navigationController presentViewController:bakerViewController animated:YES completion:nil];
            }
            else {
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:@"Wrong HPUB format" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
                [alert show];
            }
        }
        else {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:@"File book.json not found" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
        }
    }
    else {
        NSError *error = nil;
        NSData *jsonData = [NSData dataWithContentsOfFile:[issue.downloadedPath stringByAppendingPathComponent:@"book.json"]];
        NSDictionary *bookData = [NSJSONSerialization JSONObjectWithData:jsonData options:0 error:&error];
        NSArray *contents = bookData[@"contents"];
        if (contents.count > 0) {
            
            MagazinePageViewController* pageViewController = [[MagazinePageViewController alloc] init];
            
            pageViewController.htmlFiles = bookData[@"contents"];
            pageViewController.issue = issue;
            
            [self.navigationController pushViewController:pageViewController animated:YES];
//            [self.navigationController presentViewController:pageViewController animated:YES completion:nil];
        }
        else {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Whoops" message:@"No pages in this Issue" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
        }
    }
}

#pragma mark - UICollectionViewDataSource
- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return [Publisher sharedInstance].issues.count;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    IssueCell *cell = (IssueCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"IssueCell" forIndexPath:indexPath];
    cell.issue = [Publisher sharedInstance].issues[indexPath.row];
    cell.actionBlock = ^(Issue *issue) {
        [self handleIssueAction:issue];
    };
    
    return cell;
}

- (UICollectionReusableView *)collectionView:(UICollectionView *)collectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)indexPath {
    if (kind == UICollectionElementKindSectionHeader) {
        IssueCellHeader *headerView = [self.collectionView dequeueReusableSupplementaryViewOfKind:kind withReuseIdentifier:@"IssueCellHeader" forIndexPath:indexPath];
        headerView.bgImageView.image = [UIImage imageNamed:@"Header"];
        if ([[Publisher sharedInstance] isSubscribedToMagazine:nil]) {
            headerView.subscribeButton.userInteractionEnabled = NO;
            [headerView.subscribeButton setTitle:[Publisher sharedInstance].headerSubscribedText forState:UIControlStateNormal];
        }
        else {
            headerView.subscribeButton.userInteractionEnabled = YES;
            [headerView.subscribeButton setTitle:[Publisher sharedInstance].headerSubscribeText forState:UIControlStateNormal];
            if (headerView.subscribeButton.allTargets.count == 0) {
                [headerView.subscribeButton addTarget:self action:@selector(subscribeToMagazine) forControlEvents:UIControlEventTouchUpInside];
            }
        }
        
        return headerView;
    }
    
    return nil;
}

#pragma mark - UICollectionViewDelegate
- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    Issue *issue = [Publisher sharedInstance].issues[indexPath.row];
    if (issue.status == IssueStatusReady) {
        [self readIssue:issue];
    }
    else {
        [self previewIssue:issue];
    }
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout referenceSizeForHeaderInSection:(NSInteger)section {
    CGFloat height = 175.0;
    if ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad) {
        height = 250.0;
    }
    return CGSizeMake(collectionView.bounds.size.width, height);
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    CGFloat width = collectionView.bounds.size.width - 20.0;
    if ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad) {
        width = 200.0;
    }
    return CGSizeMake(width, (width-14.0)/Config.issueImageAspectRatio + 7.0 + 7.0 + 24.0 + 7.0);
}

- (UIEdgeInsets)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout insetForSectionAtIndex:(NSInteger)section {
    UIEdgeInsets insets = UIEdgeInsetsMake(10.0, 10.0, 10.0, 10.0);
    if ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad) {
        insets = UIEdgeInsetsMake(10.0, 39.0, 10.0, 39.0);
    }
    return insets;
}

#pragma mark - UIActionSheetDelegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {
    if (buttonIndex > 0) {
        NSDictionary *dict = self.menuOptions[buttonIndex - 1];
        SEL selector = NSSelectorFromString(dict[@"action"]);
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
        [self performSelector:selector];
#pragma clang diagnostic pop
    }
}

#pragma mark - UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    if (alertView.tag == tAlertDeleteIssues) {
        if (buttonIndex != alertView.cancelButtonIndex) {
            for (Issue *issue in [Publisher sharedInstance].issues) {
                [issue deleteDownload];
            }
            [self updateView];
        }
    }
}

#pragma mark - Notifications
- (void)appBecomeActive:(NSNotification *)notif {
    if (self.lastUpdateDate == nil || [[NSDate date] timeIntervalSinceDate:self.lastUpdateDate] > 60*60*24) {
        [self updateData];
    }
    
    __block BOOL prevState = NO;
    prevState = [[Publisher sharedInstance] isSubscribedToMagazine:^(BOOL subscribed, NSError *error) {
        if (subscribed != prevState) {
            [self updateView];
        }
    }];
}

@end

