//
//  PDFDocument.m
//  Almond-AppVault
//
//  Created by Roman Stetsenko on 30/12/14.
//  Copyright (c) 2014 AkmStudio Ltd. All rights reserved.
//


#import "PDFDocument.h"


@interface PDFDocument ()

+ (PDFDocument *)unarchiveFromFileName:(NSString *)filePath password:(NSString *)phrase;

@end


@implementation PDFDocument

+ (PDFDocument *)withDocumentFilePath:(NSString *)filePath password:(NSString *)phrase
{
    PDFDocument *document = nil; // ReaderDocument object
    
    document = [PDFDocument unarchiveFromFileName:filePath password:phrase];
    
    if (document == nil) // Unarchive failed so create a new ReaderDocument object
    {
        document = [[PDFDocument alloc] initWithFilePath:filePath password:phrase];
    }
    
    return document;
}

+ (PDFDocument *)unarchiveFromFileName:(NSString *)filePath password:(NSString *)phrase
{
    PDFDocument *document = nil; // ReaderDocument object
    
    NSString *fileName = [filePath lastPathComponent]; // File name only
    
    NSString *archiveFilePath = [PDFDocument archiveFilePath:fileName];
    
    @try // Unarchive an archived ReaderDocument object from its property list
    {
        document = [NSKeyedUnarchiver unarchiveObjectWithFile:archiveFilePath];
        
        if (document != nil) // Set the document's file path and password properties
        {
            [document performSelector:@selector(setFilePath:) withObject:[filePath copy]];
            [document performSelector:@selector(setPassword:) withObject:[phrase copy]];
        }
    }
    @catch (NSException *exception) // Exception handling (just in case O_o)
    {
#ifdef DEBUG
        NSLog(@"%s Caught %@: %@", __FUNCTION__, [exception name], [exception reason]);
#endif
    }
    
    return document;
}

+ (NSString *)archiveFilePath:(NSString *)fileName
{
    NSFileManager *fileManager = [NSFileManager defaultManager]; // Singleton
    
    NSString *applicationSupportPath = [PDFDocument applicationSupportPath]; // See above
    
    NSString *archivePath = [applicationSupportPath stringByAppendingPathComponent:@"Reader Metadata"];
    
    [fileManager createDirectoryAtPath:archivePath withIntermediateDirectories:NO attributes:nil error:NULL];
    
    NSString *archiveName = [[fileName stringByDeletingPathExtension] stringByAppendingPathExtension:@"plist"];
    
    return [archivePath stringByAppendingPathComponent:archiveName]; // "{archivePath}/'fileName'.plist"
}

+ (NSString *)applicationSupportPath
{
    NSFileManager *fileManager = [NSFileManager defaultManager]; // Singleton
    
    NSURL *pathURL = [fileManager URLForDirectory:NSApplicationSupportDirectory inDomain:NSUserDomainMask appropriateForURL:nil create:YES error:NULL];
    
    return [pathURL path]; // Path to the application's "~/Library/Application Support" directory
}

- (BOOL)canEmail {
    return NO;
}

- (BOOL)canExport {
    return NO;
}

- (BOOL)canPrint {
    return NO;
}

@end

