//
//  PDFReaderViewController.m
//  Almond-AppVault
//
//  Created by Roman Stetsenko on 12/28/14.
//  Copyright (c) 2014 AkmStudio Ltd. All rights reserved.
//


#import "PDFReaderViewController.h"
#import "PDFDocument.h"
#import "ReaderMainToolbar.h"


@interface PDFReaderViewController () <ReaderViewControllerDelegate>

@property (nonatomic, strong) Issue *issue;

@end


@implementation PDFReaderViewController

- (instancetype)initWithIssue:(Issue *)issue {
    PDFDocument *document = [PDFDocument withDocumentFilePath:issue.downloadedPath password:nil];
    NSAssert1(document, @"can't open document at path: %@", issue.downloadedPath);
    
    self = [super initWithReaderDocument:document];
    self.delegate = self;
    
    [ReaderConstants sharedReaderConstants].enableThumbs = NO;
    [ReaderConstants sharedReaderConstants].bookmarks = NO;
    [ReaderConstants sharedReaderConstants].showShadows = NO;
    
    self.issue = issue;
    
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // set background for view under status bar
//    UIView *v = self.view.subviews.lastObject;
//    v.backgroundColor = [UIColor whiteColor];
    
    if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad){
        self.view.backgroundColor = [UIColor whiteColor];
    }else{
        self.view.backgroundColor = [UIColor blackColor];
    }
    
    for (UIView *view in self.view.subviews) {
        if ([view isKindOfClass:[ReaderMainToolbar class]]) {
            // find label
            for (UIView *view2 in view.subviews) {
                if ([view2 isKindOfClass:[UILabel class]]) {
                    UILabel *label = (UILabel *)view2;
                    label.text = self.issue.title;
                    
                    CGFloat x2 = label.superview.bounds.size.width - CGRectGetMaxX(label.frame);
                    if (label.frame.origin.x > x2) {
                        x2 = label.frame.origin.x;
                    }
                    
                    label.frame = CGRectMake(x2, label.frame.origin.y, label.superview.bounds.size.width - 2*x2, label.frame.size.height);
                }
            }
        }
    }
}

//- (BOOL)prefersStatusBarHidden {
//    return NO;
//}

#pragma mark - ReaderViewControllerDelegate
- (void)dismissReaderViewController:(ReaderViewController *)viewController {
    [self dismissViewControllerAnimated:YES completion:nil];
}

@end

