//
//  IAPProduct.m
//  Almond-AppVault
//
//  Created by Roman Stetsenko on 11/12/14.
//  Copyright (c) 2014 AkmStudio Ltd. All rights reserved.
//


#import "IAPProduct.h"


@interface IAPProduct () <SKProductsRequestDelegate>

@property (nonatomic, strong) NSString *productId;
@property (nonatomic, assign) IAPType type;

//@property (nonatomic, strong) SKProduct *skProduct;
@property (nonatomic, copy) void(^getSKProductCompletion)(NSError *);

@end


@implementation IAPProduct

- (instancetype)initWithId:(NSString *)productId type:(IAPType)type {
    self = [super init];
    self.productId = productId;
    self.type = type;
    
    return self;
}

- (void)getSKProduct:(void (^)(NSError *))completion {
    self.getSKProductCompletion = completion;
    SKProductsRequest *req = [[SKProductsRequest alloc] initWithProductIdentifiers:[NSSet setWithObject:self.productId]];
    req.delegate = self;
    [req start];
}

- (NSString *)description {
    return [NSString stringWithFormat:@"<%@: %p> id:%@, type:%lu, title:%@, price:%@", self.class, self, self.productId, (long)self.type, self.skProduct.localizedTitle, self.skProduct.price];
}

#pragma mark - SKProductsRequestDelegate
- (void)productsRequest:(SKProductsRequest *)request didReceiveResponse: (SKProductsResponse *)response {
    NSError *error = nil;
    if (response.invalidProductIdentifiers.count > 0) {
        error = [NSError errorWithDomain:@"IAP" code:-1 userInfo:@{NSLocalizedDescriptionKey:[NSString stringWithFormat:@"Invalid identifier: %@", response.invalidProductIdentifiers[0]]}];
    }
    
    if (response.products.count > 0) {
        self.skProduct = response.products[0];
    }
    else {
        error = [NSError errorWithDomain:@"IAP" code:-1 userInfo:@{NSLocalizedDescriptionKey:[NSString stringWithFormat:@"invalid product identifier %@", self.productId]}];
    }
    RUN_BLOCK_SAFE(self.getSKProductCompletion, error);
}

- (void)request:(SKRequest *)request didFailWithError:(NSError *)error {
    if (error.code == 0) {
        error = cErrorDummy;
    }
    RUN_BLOCK_SAFE(self.getSKProductCompletion, error);
}

@end

