//
//  Publisher.m
//  Almond-AppVault
//
//  Created by Roman Stetsenko on 17/12/14.
//  Copyright (c) 2014 AkmStudio Ltd. All rights reserved.
//


#import "Publisher.h"
#import "IAPManager.h"
#import "Config.h"
#import "RNEncryptor.h"
#import "RNDecryptor.h"


@interface Publisher ()

@property (nonatomic, strong) NSString *magazineName;
@property (nonatomic, strong) NSString *headerSubscribeText;
@property (nonatomic, strong) NSString *headerSubscribedText;
@property (nonatomic, strong) NSArray *subscriptions;

@property (nonatomic, strong) NSArray *issues;

@property (nonatomic, strong) NSDate *endSubscriptionDate;

@property (nonatomic, strong) IAPManager *iapManager;

@end


@implementation Publisher

@synthesize endSubscriptionDate = _endSubscriptionDate;

+ (instancetype)sharedInstance {
    static Publisher *_sharedInstance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedInstance = [[Publisher alloc] init];
        _sharedInstance.iapManager = [[IAPManager alloc] init];
    });
    
    return _sharedInstance;
}

- (void)getLatestInfo:(void(^)(NSError *error))completion {
    NSURLRequest *request = [NSURLRequest requestWithURL:Config.jsonUrl];
    [NSURLConnection sendAsynchronousRequest:request queue:[NSOperationQueue mainQueue] completionHandler:^(NSURLResponse *response, NSData *data, NSError *connectionError) {
        NSError *error = connectionError;
        NSDictionary *dict = nil;
        if (error == nil) {
            dict = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
            if (error == nil) {
                [[NSUserDefaults standardUserDefaults] setObject:dict forKey:@"lastDict"];
            }
        }
        
        if (error != nil) {
            dict = [[NSUserDefaults standardUserDefaults] objectForKey:@"lastDict"];
            if (dict != nil) {
                error = nil;
            }
        }
        
        if (error == nil) {
            if ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad) {
                self.magazineName = dict[@"magazineTitleIpad"];
                self.headerSubscribeText = dict[@"headerSubscribeTextIpad"];
                self.headerSubscribedText = dict[@"headerSubscribedTextIpad"];
            }
            else {
                self.magazineName = dict[@"magazineTitle"];
                self.headerSubscribeText = dict[@"headerSubscribeText"];
                self.headerSubscribedText = dict[@"headerSubscribedText"];
            }
            
            NSMutableArray *subscriptions = [NSMutableArray array];
            NSArray *array = dict[@"subscriptionIds"];
            for (NSString *s in array) {
                IAPProduct *product = [[IAPProduct alloc] initWithId:s type:IAPTypeSubscriptionNonRenewable];
                [subscriptions addObject:product];
            }
            
            array = dict[@"autoSubscriptionIds"];
            for (NSString *s in array) {
                IAPProduct *product = [[IAPProduct alloc] initWithId:s type:IAPTypeSubscriptionAutoRenewable];
                [subscriptions addObject:product];
            }
#warning RS: Add free subscription support
            self.subscriptions = [subscriptions copy];
            
            
            //that is ugly but didn't found another solution
            NSMutableArray *issueProducts = [NSMutableArray array];
            for (NSDictionary *issueDict in dict[@"issues"]) {
                IAPProduct *product = [[IAPProduct alloc] initWithId:issueDict[@"in-app-purchase-id"] type:IAPTypeNonConsumable];
                [issueProducts addObject:product];
            }
            
            [issueProducts addObjectsFromArray:self.subscriptions];
            self.iapManager = [[IAPManager alloc] init];
            self.iapManager.products = [issueProducts copy];
            
            NSMutableArray *issues = [NSMutableArray array];
            [issueProducts removeAllObjects];
            for (NSDictionary *issueDict in dict[@"issues"]) {
                Issue *issue = [[Issue alloc] initWithDictionary:issueDict];
                [issues addObject:issue];
                [issueProducts addObject:issue.product];
            }
            self.issues = [issues copy];
            
            [issueProducts addObjectsFromArray:self.subscriptions];
            self.iapManager.products = [issueProducts copy];
            [self.iapManager getSKProducts:^(NSError *error) {
                RUN_BLOCK_SAFE(completion, error);
            }];
        }
        else {
            RUN_BLOCK_SAFE(completion, error);
        }
    }];
}

#pragma mark -
//- (BOOL)isSubscribedToMagazine {
//#if TARGET_IPHONE_SIMULATOR
//    return YES;
//#endif
//    
//    if (self.endSubscriptionDate != nil && [self.endSubscriptionDate compare:[NSDate date]] == NSOrderedDescending) {
//        return YES;
//    }
//    return NO;
//}

- (BOOL)isSubscribedToMagazine:(void (^)(BOOL subscribed, NSError *error))completion {
#if TARGET_IPHONE_SIMULATOR
    RUN_BLOCK_SAFE(completion, YES, nil);
    return YES;
#endif
    
    if (self.endSubscriptionDate != nil && [self.endSubscriptionDate compare:[NSDate date]] == NSOrderedDescending) {
        RUN_BLOCK_SAFE(completion, YES, nil);
        return YES;
    }
    
    if (completion != nil) {
    [self.iapManager isSubscribedNow:^(BOOL subscribed, NSDate *endDate, NSError *error) {
        if (error == nil) {
            if (subscribed) {
                self.endSubscriptionDate = endDate;
            }
            [self updateIssuesPurchaseState];
        }
        RUN_BLOCK_SAFE(completion, subscribed, error);
    }];
    }
    return NO;
}

- (void)subscribe:(IAPProduct *)subscription completion:(void (^)(NSError *))completion {
    [self.iapManager isSubscribedNow:^(BOOL subscribed, NSDate *endDate, NSError *error) {
        if (error == nil && subscribed) {
            self.endSubscriptionDate = endDate;
            [self updateIssuesPurchaseState];
            RUN_BLOCK_SAFE(completion, error);
            return;
        }
        
        [self.iapManager purchaseProduct:subscription completion:^(NSError *error) {
            if (error == nil) {
                [self.iapManager isSubscribedNow:^(BOOL subscribed, NSDate *endDate, NSError *error) {
                    if (error == nil) {
                        if (subscribed) {
                            self.endSubscriptionDate = endDate;
                        }
                        [self updateIssuesPurchaseState];
                    }
                    RUN_BLOCK_SAFE(completion, error);
                }];
            }
            else {
                RUN_BLOCK_SAFE(completion, error);
            }
        }];
    }];
}

#pragma mark - Issues
- (BOOL)isPurchasedIssue:(Issue *)issue {
    
    BOOL result = NO;
    if (!issue.paidContent ||
        (issue.paidContent && [self.iapManager isPurchased:issue.product]) ||
        (issue.paidContent && [self isSubscribedToMagazine:nil] && !issue.paidIfSubscribed)) {
        result = YES;
    }
    
    return result;
}

- (void)purchaseIssue:(Issue *)issue completion:(void (^)(NSError *))completion {
    [self.iapManager purchaseProduct:issue.product completion:^(NSError *error) {
        if (error == nil) {
            issue.status = IssueStatusNotDownloaded;
        }
        RUN_BLOCK_SAFE(completion, error);
    }];
}

- (void)restorePurchases:(void (^)(NSError *))completion {
    [self.iapManager restorePurchases:^(NSError *error) {
        if (error == nil) {
            [self.iapManager isSubscribedNow:^(BOOL subscribed, NSDate *endDate, NSError *error) {
                if (error == nil) {
                    if (subscribed) {
                        self.endSubscriptionDate = endDate;
                    }
                    [self updateIssuesPurchaseState];
                }
                RUN_BLOCK_SAFE(completion, error);
            }];
        }
        else {
            RUN_BLOCK_SAFE(completion, error);
        }
    }];
}

- (void)updateIssuesPurchaseState {
    for (Issue *issue in self.issues) {
        if (issue.status == IssueStatusNotPurchased && [self isPurchasedIssue:issue]) {
            issue.status = IssueStatusNotDownloaded;
        }
        if (issue.status != IssueStatusNotPurchased && ![self isPurchasedIssue:issue]) {
            issue.status = IssueStatusNotPurchased;
        }
    }
}

#pragma mark - Properties
- (NSString *)magazineName {
    if (_magazineName == nil) {
        return [Config magazineName];
    }
    
    return _magazineName;
}

- (NSString *)headerSubscribeText {
    if (_headerSubscribeText == nil) {
        return [Config headerSubscribeText];
    }
    
    return _headerSubscribeText;
}

- (NSString *)headerSubscribedText {
    if (_headerSubscribedText == nil) {
        return [Config headerSubscribedText];
    }
    
    return _headerSubscribedText;
}

- (void)setEndSubscriptionDate:(NSDate *)endSubscriptionDate {
    _endSubscriptionDate = endSubscriptionDate;
    
    for (Issue *issue in self.issues) {
        if (issue.status == IssueStatusNotPurchased) {
            if ([self isPurchasedIssue:issue]) {
                issue.status = IssueStatusNotDownloaded;
            }
        }
    }
    
    [self saveObject:_endSubscriptionDate forKey:@"ertse"];
}

- (NSDate *)endSubscriptionDate {
    if (_endSubscriptionDate == nil) {
        _endSubscriptionDate = [self loadObjectForKey:@"ertse"];
    }
    return _endSubscriptionDate;
}

//- (void)setLastUpdateDate:(NSDate *)lastUpdateDate {
//    _lastUpdateDate = lastUpdateDate;
//    [self saveObject:_lastUpdateDate forKey:@"esdfs"];
//}
//
//- (NSDate *)lastUpdateDate {
//    if (_lastUpdateDate == nil) {
//        _lastUpdateDate = [self loadObjectForKey:@"esdfs"];
//    }
//    return _lastUpdateDate;
//}

#pragma mark - Helpers
- (void)saveObject:(id<NSCopying>)object forKey:(NSString *)key {
    if (object == nil) {
        [[NSUserDefaults standardUserDefaults] removeObjectForKey:key];
    }
    else {
        NSData *data = [NSKeyedArchiver archivedDataWithRootObject:object];
        NSError *error;
        data = [RNEncryptor encryptData:data withSettings:kRNCryptorAES256Settings password:[NSString stringWithFormat:@"sf2w1d%@d3ssdf%.faka", [NSStringFromClass([NSMutableArray class]) substringWithRange:NSMakeRange(4, 6)], [Config ver]] error:&error];
        NSAssert1(error == nil, @"encrypt error %@", error);
        [[NSUserDefaults standardUserDefaults] setObject:data forKey:key];
    }
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (id)loadObjectForKey:(NSString *)key {
    NSError *error = nil;
    NSData *data = [[NSUserDefaults standardUserDefaults] objectForKey:key];
    if (data == nil) {
        return nil;
    }
    data = [RNDecryptor decryptData:data withSettings:kRNCryptorAES256Settings password:[NSString stringWithFormat:@"sf2w1d%@d3ssdf%.faka", [NSStringFromClass([NSMutableArray class]) substringWithRange:NSMakeRange(4, 6)], [Config ver]] error:&error];
    NSAssert1(error == nil, @"decrypt error %@", error);
    id obj = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    return obj;
}

@end

